#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#

(define-module (ordo action quadlet)
  #:use-module (ice-9 filesystem)
  #:use-module (ini)
  #:use-module (logging logger)
  #:use-module (ordo action remote-cmd)
  #:use-module ((ordo action filesystem) #:prefix fs:)
  #:use-module ((srfi srfi-1) #:select (remove))
  #:export (create-network
            create-pod
            create-container
            create-volume
            create-image
            create-build))

(define system-quadlet-dir "/etc/containers/systemd")

(define default-install-options '(("WantedBy" . "multi-user.target default.target")))

(define (scm->ini-string data)
  (with-output-to-string (lambda () (scm->ini data))))

(define (quadlet quadlet-type name description unit-options quadlet-options service-options install-options)
  (let* ((description (or description (string-append "Podman " (string-downcase quadlet-type) " " name)))
         (data  `(("Unit" ("Description" . ,description) ,@unit-options)
                  (,(string-titlecase quadlet-type) ,@quadlet-options)
                  ,@(if (null? service-options) '() (list (cons "Service" service-options)))
                  ,@(if (null? install-options) '() (list (cons "Install" install-options))))))
    (scm->ini-string data)))

(define* (create-network conn #:key name description network (unit '()) (service '()) (install default-install-options) (quadlet-dir system-quadlet-dir))
  (fs:install-file conn
                   #:path (file-name-join* quadlet-dir (string-append name ".network"))
                   #:content (quadlet "Network" name description unit network service install)))

(define* (create-pod conn #:key name description pod (unit '()) (service '()) (install default-install-options) (quadlet-dir system-quadlet-dir))
  (fs:install-file conn
                   #:path (file-name-join* quadlet-dir (string-append name ".pod"))
                   #:content (quadlet "Pod" name description unit pod service install)))

(define* (create-container conn #:key name description container (unit '()) (service '()) (install default-install-options) (quadlet-dir system-quadlet-dir))
  (fs:install-file conn
                   #:path (file-name-join* quadlet-dir (string-append name ".container"))
                   #:content (quadlet "Container" name description unit container service install)))

(define* (create-volume conn #:key name description volume (unit '()) (service '()) (install '()) (quadlet-dir system-quadlet-dir))
  (fs:install-file conn
                   #:path (file-name-join* quadlet-dir (string-append name ".volume"))
                   #:content (quadlet "Volume" name description unit volume service install)))

(define* (create-build conn #:key name description build (unit '()) (service '()) (install '()) (quadlet-dir system-quadlet-dir))
  (fs:install-file conn
                   #:path (file-name-join* quadlet-dir (string-append name ".build"))
                   #:content (quadlet "Build" name description unit build service install)))

(define* (create-image conn #:key name description image (unit '()) (service '()) (install '()) (quadlet-dir system-quadlet-dir))
  (fs:install-file conn
                   #:path (file-name-join* quadlet-dir (string-append name ".image"))
                   #:content (quadlet "Image" name description unit image service install)))
